/*********************************************************************
*                                                                    *          
*                              FLY                                   *
*                           --E#Y#E--				     *
*                         ===MUSCLE===                               *
*                                                                    *
*********************************************************************/
/* Header Files */
#include "win.h"

/* Define Time */
#define DATE 75

/*---------------------------------------------------------------------------*/

/* Font Position */
static char bitfont[] = 
{"-bitstream-bitstream charter-bold-r-normal--17-120-100-100-p-105-iso8859-1"};

/* Display Variables */
static unsigned int width;
static unsigned int height;
static Display *display;
static Window monitor;
static int depth;

/* Display Attributes */
GLint                   att[] = { GLX_USE_GL,
				  GLX_RGBA,
				  GLX_RED_SIZE, 4,
				  GLX_GREEN_SIZE, 4,
				  GLX_BLUE_SIZE, 4,
				  GLX_ALPHA_SIZE, 4,
				  GLX_AUX_BUFFERS,  0,
				  GLX_ACCUM_RED_SIZE, 0,
				  GLX_ACCUM_GREEN_SIZE, 0,
				  GLX_ACCUM_BLUE_SIZE, 0, 
				  GLX_ACCUM_ALPHA_SIZE, 0,
				  GLX_BUFFER_SIZE, 24,
				  GLX_DEPTH_SIZE, 24,
				  GLX_STENCIL_SIZE, 8,
				  GLX_DOUBLEBUFFER, None};

/* Display Context */
GC 			mono;
Font 			font;
XVisualInfo             *vi;
Colormap                cmap;
XColor			color;
XSetWindowAttributes    swa;
Window                  windows;
GLXContext              glc;
XWindowAttributes       gwa;
XEvent                  xev;

/*-------------------------------------------------------------------------*/

/* Create Window */
void createWin(char wide)
{	
	/* Window-Name */
	const char *sname = "[ LANDSCAPE - 1.4 ]";
	const char *nicon = "[ LANDSCAPE - 1.4 ]";
	
	/* Connect to X-Server */
	if((display = XOpenDisplay(NULL)) == NULL) 
	{
		printf("\nCannot connect to x server\n\n");
		exit(0);
	}

	/* Default Window */
	monitor = XDefaultRootWindow(display);
	
	/* Choose Visual */ 
	if((vi = glXChooseVisual(display, 0, att)) == NULL) 
	{
		printf("\nNo matching visual\n\n");
		exit(0);
	}
      
	/* Default Depth */
    	depth = XDefaultDepth(display, monitor);
	//color = XDefaultColormap(display, monitor);
        
	/* Create Colormap */
	if((cmap = XCreateColormap(display, monitor, vi->visual, AllocNone)) == 0) 
	{
		printf("\nCannot create colormap\n\n");
		exit(0);
	}
	
	/* Screen */
	if(wide)
	{
		/* Fullscreen Screen */
		width = XDisplayWidth(display, 0);
		height = XDisplayHeight(display, 0);
		swa.override_redirect = 1;
		XGrabKeyboard(display, monitor, 0, 
		GrabModeAsync, GrabModeAsync, CurrentTime);
	}
	else
	{
		/* Standard Screen */
		width = 800;
		height = 600;
		swa.override_redirect = 0;
	}

	/* Max-Min Size */
	static XSizeHints sizeh;
	sizeh.flags = PSize | PMinSize | PMaxSize;
	sizeh.min_width = width;
	sizeh.max_width = width;
	sizeh.min_height = height;
	sizeh.max_height = height;

	/* Create Colormap */
	cmap = XCreateColormap(display, monitor, vi->visual, AllocNone);
	swa.colormap = cmap;
	swa.event_mask =  KeyPressMask | KeyReleaseMask | ButtonPressMask | 
			ButtonReleaseMask | EnterWindowMask | LeaveWindowMask;

	/* Create Window */
	windows = XCreateWindow(display,monitor,0,0,width,height,2, 
				vi->depth, InputOutput, vi->visual,
				CWBorderPixel | CWOverrideRedirect | 
				CWEventMask | CWColormap , &swa);	
	
	XSetStandardProperties(display,windows,sname,nicon,None,0,0,&sizeh);
	XStoreName(display, windows, "[ LANDSCAPE - 1.4 ]");
	
	/* Map Window */
	XMapWindow(display, windows);	
	glc = glXCreateContext(display, vi, NULL , GL_TRUE);
	glXMakeCurrent(display, windows, glc);
}

/*-------------------------------------------------------------------------*/

/* Close Window */
void closeWin()
{
	glXMakeCurrent(display, None, NULL);
	glXDestroyContext(display, glc);
	XDestroyWindow(display, windows);
	XCloseDisplay(display);
	exit(0);
}

/*-------------------------------------------------------------------------*/

/* Look / View */
void perspective() 
{	
	/* Viewport*/
	XGetWindowAttributes(display, windows, &gwa);
	glViewport(0, 0, gwa.width, gwa.height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	/* Perspective */
	gluPerspective(52.0f, (float)(gwa.width) 
	/ (float)(gwa.height), 1.0f, 100.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glXSwapBuffers(display, windows);

	/* Screen Resolution */
	printf("%dx%d screen ", gwa.width, gwa.height);
}

/*-------------------------------------------------------------------------*/

/* X-Load Font */
void loadFont()
{
	/* Date and Time */
	char mdate[DATE];
	struct tm *date;
	time_t ndate;
	time(&ndate);
	date = localtime(&ndate);

	/* Font Variables */
	char muser[14];
	char screen[14];
	char frames[14];
	int le = 12;

	/*  Load Font and Color */	
	mono = XCreateGC(display, windows,0,0);
	font = XLoadFont(display, bitfont); 
	XSetFont(display, mono, font);	

	/* Draw Font */
	strftime(mdate, DATE,"   %X", date);
	sprintf(muser,"[ Vulkano ]:");
	sprintf(screen," %4d x%4d", gwa.width, gwa.height);
	sprintf(frames,"  %4.2f FPS", FPS);
	XSetForeground(display, mono , 248248255);
	XDrawString(display, windows, mono ,11, 25, mdate, le);
	XSetForeground(display, mono , 100100100);
	XDrawString(display, windows, mono ,111,25, muser, le);
	XSetForeground(display, mono , 248248255);
	XDrawString(display, windows, mono ,width - 120,25, frames, le);
	XDrawString(display, windows, mono ,width - 120,50, screen, le);

	/* Free Memory */
	XFlush(display);
	XFreeGC(display, mono);
}

/*-------------------------------------------------------------------------*/

/* Event Loop */
void eventloop() 
{ 	
	/* Events */
	XEvent xev;
	int events;
	XFlush(display);
	events = XPending(display);
	while (( events != 0 )) 
	{
		/* Events */
		events--;
		XNextEvent(display, &xev);
		process_event(xev);
	}
}

/*---------------------------------------------------------------------------*/

